# #############################################################################
# Copyright 2024 F. Hoffmann-La Roche
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# #############################################################################
from .table_summary import (calculate_table_summary, 
        categorical_n, categorical_n_percent, categorical_percent, 
        numerical_mean_sd, numerical_median_iqr, numerical_median_q1q3, numerical_min_max, 
        numerical_missing)
from .sample_data import get_sample_data
from .reportable import pandas_to_report_html, get_styles, Pandas2HTMLSummaryTable
from .pysummaries import get_table_summary

__all__ = ['get_table_summary',
        'calculate_table_summary',
        'categorical_n', 'categorical_n_percent', 'categorical_percent',
        'numerical_mean_sd', 'numerical_median_iqr', 'numerical_median_q1q3', 'numerical_min_max', 
        'numerical_missing', 'get_sample_data', 'pandas_to_report_html', 'get_styles', 'Pandas2HTMLSummaryTable']

__version__ = '0.0.1b1'



# #############################################################################
# Copyright 2024 F. Hoffmann-La Roche
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# #############################################################################
"""
Functions to summarise pandas series
"""
import pandas as pd

def categorical_n(curseries, rounding):
    """
    Calculates the N for each category in the series.

    :param curseries: series to be summarized
    :type curseries: pandas series
    :param rounding: number of decimal points to show round the results
    :type rounding: int
    :return: a series with a numerical or string value per categorical level
    :rtype: pandas series
    """
    curstat_n = len(curseries)
    dosort = True
    if curseries.dtype.name == 'category':
        dosort = False
    curn = curseries.value_counts(sort=dosort)
    return curn

def categorical_n_percent(curseries, rounding):
    """
    Calculates "N (%)" fo each category in the series

    :param curseries: series to be summarized
    :type curseries: pandas series
    :param rounding: number of decimal points to show round the results
    :type rounding: int
    :return: a series with a numerical or string value per categorical level
    :rtype: pandas series
    """
    curstat_n = len(curseries)
    dosort = True
    if curseries.dtype.name == 'category':
        dosort = False
    curperc = curseries.value_counts(sort=dosort).div(float(curstat_n)).mul(100)
    if rounding is not None:
        curperc = round(curperc, rounding)
    curperc = " (" + curperc.astype(str).str.cat([" %)"]*len(curperc))
    curn = curseries.value_counts(sort=dosort).astype(str).str.cat(curperc)
    return curn

def categorical_percent(curseries, rounding):
    """
    Calculates the percentage for each category in the series

    :param curseries: series to be summarized
    :type curseries: pandas series
    :param rounding: number of decimal points to show round the results
    :type rounding: int
    :return: a series with a numerical or string value per categorical level
    :rtype: pandas series
    """
    curstat_n = len(curseries)
    curperc = curseries.value_counts().div(float(curstat_n)).mul(100)
    if rounding is not None:
        curperc = round(curperc, rounding)
    curperc = curperc.astype(str).str.cat([" %"]*len(curperc))
    return curperc

def numerical_mean_sd(curseries, rounding):
    """
    Calculates "Mean (SD)" for the numerical series

    :param curseries: series to be summarized
    :type curseries: pandas series
    :param rounding: number of decimal points to show round the results
    :type rounding: int
    :return: a single value with the summary for the series
    :rtype: int, float or string
    """
    mean = str(round(curseries.mean(), 1))
    std = curseries.std()
    if rounding is not None:
        std = round(std, rounding)
    std = " (" + str(std) + ")"
    return mean + std

def numerical_median_iqr(curseries, rounding):
    """
    Calculates "Median [IQR]" for the numerical series

    :param curseries: series to be summarized
    :type curseries: pandas series
    :param rounding: number of decimal points to show round the results
    :type rounding: int
    :return: a single value with the summary for the series
    :rtype: int, float or string
    """
    median = curseries.median()
    iqr = curseries.quantile(0.75) - curseries.quantile(0.25)
    if rounding is not None:
        median = round(median, rounding)
        iqr = round(iqr, rounding)
    median = str(median)
    iqr = " [" + str(iqr) + "]"
    return median + iqr

def numerical_median_q1q3(curseries, rounding):
    """
    Calculates "Median [Q1 ; Q3]" for the numerical series

    :param curseries: series to be summarized
    :type curseries: pandas series
    :param rounding: number of decimal points to show round the results
    :type rounding: int
    :return: a single value with the summary for the series
    :rtype: int, float or string
    """
    median = curseries.median()
    q1 = curseries.quantile(0.25)
    q3 = curseries.quantile(0.75)
    if rounding is not None:
        median = round(median, rounding)
        q1 = round(q1, rounding)
        q3 = round(q3, rounding)
    median = str(median)
    iqr = " [" + str(q1) +  " ; " + str(q3) + "]"
    return median + iqr

def numerical_min_max(curseries, rounding):
    """
    Calculates "Min ; Max" for the numerical series

    :param curseries: series to be summarized
    :type curseries: pandas series
    :param rounding: number of decimal points to show round the results
    :type rounding: int
    :return: a single value with the summary for the series
    :rtype: int, float or string
    """
    minimum = curseries.min()
    maximum = curseries.max()
    if rounding is not None:
        minimum = round(minimum, rounding)
        maximum = round(maximum,rounding)
    minimum = str(minimum)
    maximum = str(maximum)
    return minimum + " ; " + maximum

def numerical_missing(curseries, rounding):
    """
    Calculates "N (%)" of missing values (na) for the numerical series

    :param curseries: series to be summarized
    :type curseries: pandas series
    :param rounding: number of decimal points to show round the results
    :type rounding: int
    :return: a single value with the summary for the series
    :rtype: int, float or string
    """
    n = 0
    perc = 0
    if len(curseries):
        n = len(curseries[pd.isna(curseries)])
        perc = 100 * (n/len(curseries)) 
    if rounding is not None:
        perc = round(perc, 1)
    return str(n) + " (" + str(perc) + " %)"





# #############################################################################
# Copyright 2024 F. Hoffmann-La Roche
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# #############################################################################

import pandas as pd

from .utils import detect_df_col_types
from . import summary_fun as sf


# TODO: 
# re-index
# calculate statistics by row
# control overall position end or start
# be able to specify column type
#
# optional:
# drop or insert levels according to number of functions

### 
# functions presets

categorical_presets = {
        'n_percent': (sf.categorical_n_percent, '0 (0%)'), 
        'n': (sf.categorical_n, 0),
        'percent': (sf.categorical_percent, '0%'),
}

numerical_presets = {
    'meansd_medianq1q3_minmax_missing': {"Mean (SD)": sf.numerical_mean_sd,
                            "Median [Q1 ; Q3]": sf.numerical_median_q1q3,
                            "Min ; Max" : sf.numerical_min_max,
                            "Missing": sf.numerical_missing,
                            },
    'meansd_medianiqr_minmax_missing': {"Mean (SD)": sf.numerical_mean_sd,
                            "Median [IQR]": sf.numerical_median_iqr,
                            "Min ; Max" : sf.numerical_min_max,
                            "Missing": sf.numerical_missing,
                            },

}

def calculate_stats(df, var, functions, coltype, strata=None, stratcat=None, var_label=None, rounding=1, categorical_missing_level=None):
    """
    For the dataframe df, for the variable var, apply all the functions. 
    If there is a stratum stratcat defined, then the dataframe will be sliced
    using the column strata for the stratum defined.
    If var_label is not None, then it will be used as index for the returning dataframe
    """
    if strata is None and stratcat is None:
        curseries = df.loc[:, var]
    else:
        curseries = df.loc[df[strata]==stratcat, var]
    if coltype=='categorical' and  categorical_missing_level:
        curseries = curseries.copy()
        if curseries.dtype.name=='category':
            cats = curseries.cat.categories.to_list() + [categorical_missing_level]
            curseries = curseries.cat.set_categories(cats)
        curseries = curseries.fillna(categorical_missing_level)
    curstratdf = None
    for funlabel, fun in functions.items():
        curstat = fun(curseries, rounding)
        if coltype == "categorical":
            if var_label:
                curstat.index = pd.MultiIndex.from_tuples([(str(var_label), str(a)) for a in curstat.index])
            else:
                curstat.index = pd.MultiIndex.from_tuples([(str(var), str(a)) for a in curstat.index])
        elif coltype == "numerical":
            if type(curstat) != pd.Series:
                curstat = pd.Series(curstat)
            if var_label:
                curstat.index = pd.MultiIndex.from_tuples([(str(var_label), str(funlabel))])
            else:
                curstat.index = pd.MultiIndex.from_tuples([(str(var), str(funlabel))])
        if curstratdf is None:
            curstratdf = curstat
        else: 
            curstratdf = pd.concat([curstratdf, curstat])
    return curstratdf


def calculate_table_summary(df, strata=None, show_overall=True, columns_labels=None, overall_name='Overall',
        columns_include=None, columns_exclude=None,
        categorical_functions=None, numerical_functions=None, rounding=1, 
        categorical_missing_level='Missing'):
    """
    Calculates  a table summary from a pandas dataframe.

    :param df: pandas dataframe from which to calculate the table one
    :type df: pandas dataframe, mandatory
    :param strata: the name of a column in the dataframe to stratify the table one (columns)
    :type strata: str, optional
    :param show_overall: Show the Overall column. By default True. If False it will take effect only if strata is defined, otherwise ignored
    :type show_overall: bool, optional
    :param columns_labels: A dictionary defining labels for the columns. Keys should be the column name and values a string with the label. Non existing
        columns will be ignored.
    :type columns_labels: dict, optional
    :param overall_name: name for the column Overall
    :type overall_name: string, optional
    :param columns_include: columns to include in the report, also determines the order of columns in the report
    :type columns_include: list, optional
    :param columns_exclude: columns to exclude from the report
    :type columns_exclude: list, optional
    :param categorical_functions: if a string, one of the presets for categorical summarization will be applied, by default n_percent. If a tuple or list, 
        the first element must be a function to apply to categorical functions, the second element must be a string or number with the value to report if there are no 
        elements in a categorical level (for example 0 (0%) for n_percent. If None those will be reported as nan.
    :type categorical_functions: str,  list or tuple, optional
    :param numerical_functions: if a string, one of the presets for numerical summarization will be applied, the default one is 'meansd_medianq1q3_minmax_missing. 
        If a dictionary it should have a label (as it should appear in the rows index) and as a values 
        functions to apply to the numerical columns of the dataframe. Multiple pairs of labels and functions are supported. 
    :type numerical_functions: str or dict, optional
    :param rounding: number of decimal points to show, by default 1.
    :type rounding: int, optional
    :param categorical_missing_level: if a categorical column has NAs, they will be replaced by the string indicated here, by default 'Missing'. That will create a new level 
        in the category. If set to None, the NAs will not be replaced.
    :type categorical_missing_level: str, optional
    :return: the table summary as a pandas dataframe
    :rtype: pandas dataframe
    :return: the number of observations for each column in the table summary as a dictionary where keys are column names (strata levels) and 
      the values are the counts as integers.
    :rtype: dictionary
        
    :Example:
    
    >>> from pysummaries import calculate_table_summary
    >>> tone = calculate_table_summary(df, strata="age_groups")

    """

    if categorical_functions:
        if type(categorical_functions)==str:
            temp = categorical_presets.get(categorical_functions)
            if not temp:
                raise Exception(f"categorical preset {categorical_functions} not defined!")
            categorical_functions = temp
        elif type(categorical_functions)==list or type(categorical_functions)==tuple:
            if len(categorical_functions)!=2:
                raise Exception("The length of categorical functions must be 2!")
            if not callable(categorical_functions[0]):
                raise Exception("The first element of categorical_functions must be a function")
            #if not type(categorical_functions[1])==str:
                #raise Exception("The second element of categorical_functions must be a string")
        else:
            raise Exception("categorical_functions should be either string, list or tuple")
    else:
        categorical_functions = categorical_presets["n_percent"]

    if numerical_functions:
        if type(numerical_functions)==str:
            temp = numerical_presets.get(numerical_functions)
            if not temp:
                raise Exception(f"numerical preset {numerical_functions} not defined!")
            numerical_functions = temp
        elif type(numerical_functions)==dict:
            if not all([callable(x) for x in numerical_functions.values()]):
                raise Exception("The values of numerical_functions must be functions")
            if not all([type(x)==str for x in numerical_functions.keys()]):
                raise Exception("The keys of numerical_functions must be strings")
        else:
            raise Exception("numerical_functions should be either string or dict")
    else:
        numerical_functions = numerical_presets["meansd_medianq1q3_minmax_missing"]


    coltypes = detect_df_col_types(df)
    colnames = df.columns.to_list()
    strat_cats = list()
    if strata is not None:
        if any(pd.isna(df[strata])):
            raise Exception("strata may not contain missing values")
        if strata in coltypes and strata in colnames:
            del coltypes[strata]
            colnames.remove(strata)
        else:
            raise Exception(f"strata column {strata} not found in dataframe")
        strat_cats = df[strata].unique()

    if columns_include:
        colnames = [c for c in columns_include if c in colnames]
    if columns_exclude:
        colnames = [c for c in colnames if c not in columns_exclude]
    if not colnames:
        raise Exception("No columns left after filtering for columns_include, columns_exclude and strata")

    df_list = list()
    strat_numbers = dict()
    for colname in colnames:
        coltype = coltypes[colname]
        var_dict = dict()
        if coltype == "categorical":
            curfuns, catna = categorical_functions
            curfuns = {'': curfuns}
        elif coltype == "numerical":
            curfuns = numerical_functions
        else:
            raise NotImplementedError(f"statistics for column {colname} (coltype) not implemented")
        col_label=None
        if columns_labels:
            col_label = columns_labels.get(colname)
        # overall
        overalldf = calculate_stats(df, colname, curfuns, coltype, rounding=rounding, var_label=col_label, categorical_missing_level=categorical_missing_level)
        # strata
        for stratcat in strat_cats:
            curstratdf = calculate_stats(df, colname, curfuns, coltype, strata=strata, stratcat=stratcat, rounding=rounding, var_label=col_label, categorical_missing_level=categorical_missing_level)
            var_dict[stratcat] = curstratdf
            strat_numbers[stratcat] = len(df.loc[df[strata]==stratcat])
        var_dict[overall_name] = overalldf
        strat_numbers[overall_name] = len(df)
        var_df = pd.DataFrame(var_dict)
        if coltype == "categorical":
            if catna:
                var_df = var_df.fillna(catna)
        df_list.append(var_df)

    tonedf = pd.concat(df_list)

    if not show_overall and strata:
        tonedf = tonedf.drop(columns=overall_name)
        del strat_numbers[overall_name]

    return tonedf, strat_numbers




# #############################################################################
# Copyright 2024 F. Hoffmann-La Roche
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# #############################################################################
import pandas as pd

from .html_elements import TabSuperHeader, TabHeader, RowGroupLabel, RowLabel, RowValue, Block, Row


def extract_multiheader(cols, strat_numbers, styles, tabid, show_index=True):
    """
    Gets the dataframe columns and transforms to superheaders (most external columns in a multi-index),
    tabheaders (most internal column names) and numcols (number of columns/tabheaders)
    when the dataframe columns are a multi-index object

    :param cols: pandas dataframe column multi-index object
    :param strat_numbers: dict with index as key and int as value, to put the N= in the column header
    :param styles: dict with default css styles
    :param show_index: if True that means the user wants to show the row index and we need to insert here a placeholder in the first
           column
    :return: superheaders, tabheaders
    """
    default_superheader_style = styles.get("superheader")
    default_header_style = styles.get("header")
    superheaders = [list() for x in range(cols.nlevels-1)]
    tabheaders = []
    lastsuphead = [None] * (cols.nlevels-1)
    tabcnt = 0
    for colindx, idx in enumerate(cols):
        *suphead_titles, head_title = idx
        # last (more internal) header
        if colindx == 0 and show_index:
            tabheader = TabHeader("", tabid, tabcnt, style=default_header_style)
            tabheaders.append(tabheader)
            tabcnt +=1
        tabheader = TabHeader(head_title, tabid, tabcnt, n=strat_numbers.get(idx), style=default_header_style)
        tabheaders.append(tabheader)
        tabcnt +=1
        # outer headers
        for sh_idx, suphead_title in enumerate(suphead_titles):
            top = True if sh_idx==0 else False
            if superheaders[sh_idx]:
                supcnt = superheaders[sh_idx][-1].idnum + 1
            else:
                supcnt = 0
            # empty for first column
            if colindx == 0:
                tabsuperheader = TabSuperHeader("", tabid, sh_idx, supcnt, style=default_superheader_style, top=top)
                superheaders[sh_idx].append(tabsuperheader)
                supcnt += 1
            if suphead_title != lastsuphead[sh_idx]:
                tabsuperheader = TabSuperHeader(suphead_title, tabid, sh_idx, supcnt, style=default_superheader_style, top=top)
                superheaders[sh_idx].append(tabsuperheader)
                lastsuphead[sh_idx] = suphead_title
            else:
                superheaders[sh_idx][-1].span +=1
        for row in superheaders:
            for elem in row:
                elem.consolidate_styles()
    return superheaders, tabheaders

def extract_simpleheader(cols, strat_numbers, styles, tabid, show_index=True):
    """
    Gets the dataframe columns and transforms to superheaders (empty in this case),
    tabheaders (column names) and numcols (number of columns/tabheaders)
    when cols is not a multi-index

    :param cols: pandas dataframe columns index object
    :param strat_numbers: dict with index as key and int as value, to put the N= in the column header
    :param styles: dict with default css styles
    :param show_index: if True that means the user wants to show the row index and we need to insert here a placeholder in the first
           column
    :return: superheaders, tabheaders
    """
    default_header_style = styles.get("header")
    superheaders = []
    tabheaders = list()
    tabcnt = 0 
    if show_index:
        curtab = TabHeader("", tabid, tabcnt, style=default_header_style, top=True)
        tabheaders.append(curtab)
        tabcnt += 1
    for indx, coltitle in enumerate(cols):
        curtab = TabHeader(coltitle, tabid, tabcnt, n=strat_numbers.get(coltitle), style=default_header_style, top=True)
        tabheaders.append(curtab)
        tabcnt += 1
    return superheaders, tabheaders

def extract_headers(df, strat_numbers, styles, tabid, show_index=True): 
    """
    Gets the dataframe columns and transforms to superheaders (most external columns in a multi-index, empty if simple index),
    tabheaders (most internal column names) and numcols (number of columns/tabheaders)

    :param df: pandas dataframe
    :param strat_numbers: dict with index as key and int as value, to put the N= in the column header
    :param styles: dict with default css styles
    :param show_index: if True that means the user wants to show the row index and we need to insert here a placeholder in the first
           column
    :return: superheaders, tabheaders, numcols
    """
    cols = df.columns
    if type(cols) == pd.MultiIndex:
        superheaders, tabheaders = extract_multiheader(cols, strat_numbers, styles, tabid, show_index=show_index)
    else:
        superheaders, tabheaders = extract_simpleheader(cols, strat_numbers, styles, tabid, show_index=show_index)

    numcols = len(tabheaders)+1
    return superheaders, tabheaders, numcols

def add_row(currows, currow_label, row_padding, row_style, default_valuestyle, value_styles, cnt, rowseries, tabid, last_row=False, show_index=True):
    """
    Add a Row object to a currows list of Rows. Each Row has a label (most inner row index in the pandas dataframe)
    and a list of Values objects. It does not return anything but adds the Row to currows in place.

    :param currows: list of rows where to add The Row.
    :param currow_label: str with the label for the Row object
    :param row_padding: int with the left padding for the Row label
    :param row_style: str with css style for the row label
    :param default_valuestyle: str with css default style for Values
    :param value_sytles: list of lists with css strings with styles for values to be added to the default
    :param cnt: int, number of row
    :param rowseries: list with the numerical or string values for the row
    :param tabid: str with the table id
    :param last_row: if True a style for a bottom border will be added
    :param show_index: if True the row index will be shown
    """

    if show_index:
        rowlabelobj = RowLabel(currow_label, row_padding, tabid, cnt, last_row=last_row, style=row_style)
        rowlabelobj.consolidate_style()
    else:
        rowlabelobj = None
    currowvalues_style = [default_valuestyle for x in range(len(rowseries))]
    if  value_styles:
        currowvalues_style =  [ x+y if y else x for x,y in zip(currowvalues_style, value_styles[cnt-1])]
    rowvalues = [RowValue(x, tabid, cnt, colidx, last_row=last_row, style=s) for colidx,(x,s) in enumerate(zip(rowseries.tolist(), currowvalues_style))]
    _ = [x.consolidate_style() for x in rowvalues]
    currow = Row(rowlabelobj, rowvalues)
    currows.append(currow)

def extract_multibodyblocks(indexes, df, value_styles, styles, tabid):
    """
    Extract blocks when there are row multi-indexes. Each block
    consist of a list of titles (the most external indexes), and
    a Row object, which has a title (most internal index) and values for that row.

    :param indexes: dataframe row indexes
    :param df: pandas dataframe
    :param value_styles: list of lists with styles for values
    :param tabid: str with id for table
    :return: list of blocks for the body.
    """
    default_valuestyle = styles.get("value")
    default_rowlabelstyle = styles.get("rowlabel")
    bodyblocks = list()
    lastblock_index = None
    prevlastblock_index = None
    curblock = Block()
    currows = list()
    cnt = 1
    leftpadding = 0.5
    left_paddings = list()
    for x in range(indexes.nlevels):
        left_paddings.append(leftpadding)
        leftpadding += 2
    *labels_paddings, row_padding = left_paddings
    rowgroup_cnts = [0] * (indexes.nlevels-1)
    for rowlabel, rowseries in df.iterrows():
        #row labels
        *rowgroup_labels, currow_label = rowlabel
        if (rowgroup_labels != lastblock_index and lastblock_index is not None) or cnt==len(df):
            rowgroupstyle = styles.get("rowgrouplabel")
            titles = [RowGroupLabel(x, tabid, level, y, rowgroup_cnts, style=rowgroupstyle) for level,(x,y) in enumerate(zip(lastblock_index, labels_paddings))]
            # prune 
            if prevlastblock_index is not None:
                indxestopop = list()
                for curindx, (prev_title, cur_title) in enumerate(zip(prevlastblock_index, lastblock_index)):
                    if prev_title == cur_title:
                        indxestopop.append(curindx)
                    else:
                        break
                indxestopop.reverse()
                for x in indxestopop:
                    titles.pop(x)
            # consolidate now that we pruned
            for tit in titles:
                tit.consolidate_style()

            curblock.titles = titles
            if cnt == len(df):
                # add values if we are handling the last row
                add_row(currows, currow_label, row_padding, default_rowlabelstyle, default_valuestyle, value_styles, cnt-1, rowseries, tabid, last_row=True)
            curblock.rows = currows
            bodyblocks.append(curblock)
            curblock = Block()
            currows = list()
            prevlastblock_index = lastblock_index
        lastblock_index = rowgroup_labels
        # values
        add_row(currows, currow_label, row_padding, default_rowlabelstyle, default_valuestyle, value_styles, cnt-1, rowseries, tabid, last_row=False)
        cnt +=1 
    return bodyblocks

def extract_simplebodyblocks(indexes, df, value_styles, styles, tabid, show_index=True):
    """
    Extract blocks when there is a simple index (not multi-index). The object returned
    has only one block consisting of 
    Row objects, each having a title (most internal index) and values for that row.

    :param indexes: dataframe row indexes
    :param df: pandas dataframe
    :param value_styles: list of lists with styles for values
    :param tabid: str with id for table
    :param show_index: wether to show the index
    :return: list of blocks for the body.
    """
    default_valuestyle = styles.get("value")
    default_rowlabelstyle = styles.get("rowlabel")
    curblock = Block()
    curblock.titles = list()
    currows = list()
    row_padding = 0.5
    cnt = 1
    last_row = False
    for rowlabel, rowseries in df.iterrows():
        if cnt == len(df):
            last_row = True
        add_row(currows, rowlabel, row_padding, default_rowlabelstyle, default_valuestyle, value_styles, cnt-1, rowseries, tabid, last_row=last_row, show_index=show_index)
        cnt += 1
    curblock.rows = currows
    bodyblocks = [curblock]
    return bodyblocks


def extract_bodyblocks(df, value_styles, styles, tabid, show_index=True):
    """
    Extract blocks from the dataframe. The object returned is a list of blocks, 
    one block consisting of a list of titles (most external row indexes) and a Row 
    Row object, each having a title (most internal index) and values for that row.

    :param df: pandas dataframe
    :param value_styles: list of lists with styles for values
    :param tabid: str with id for table
    :param show_index: wether to show the index. This is effective only if the dataframe has a simple
           (non-multiindex) index
    :return: list of blocks for the body.
    """
    # body
    indexes = df.index

    if type(indexes) == pd.MultiIndex and show_index:
        bodyblocks = extract_multibodyblocks(indexes, df, value_styles, styles, tabid)
    else:
        bodyblocks = extract_simplebodyblocks(indexes, df, value_styles, styles, tabid, show_index=show_index)

    return bodyblocks


def df_to_intermediate_rep(df, tabid, strat_numbers=None, value_styles=None, styles=None, show_index=True):
    """
    Transform a pandas dataframe to an intermediate representation consisting of superheaders (list of most external
    column indexes if multi-index else empty list), tabheaders (list of most internal column indexes), numcols
    (number of columns), and bodyblocks (object with a list of labels (most external row indexes) and a Row
    object consisting of a row label (most internal row index) and values)

    :param df: pandas dataframe
    :param tabid: str with the id for the table
    :param strat_numbers: dict with keys being column index and value numbers to insert in the header as N=xx
    :param value_styles: list of lists with css style for values
    :param styles: dict with default styles
    :param show_index: if True the row index must be shown, makes sense only if row index is not multi-index
    :return: superheaers, tabheaders, numcols, bodyblocks
    """

    superheaders, tabheaders, numcols = extract_headers(df, strat_numbers, styles, tabid, show_index=show_index)
    bodyblocks = extract_bodyblocks(df, value_styles, styles, tabid, show_index=show_index)
    return superheaders, tabheaders, numcols, bodyblocks




# #############################################################################
# Copyright 2024 F. Hoffmann-La Roche
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# #############################################################################

default_styles = {"table": 'font-family: Arial, sans-serif; border-collapse: collapse; padding: 0px; margin: 0px;',
          "caption": 'padding: 0px; margin: 0px; text-align: left !important; margin-bottom: 5px;',
          "superheader": 'padding: 0.5ex 1.5ex; margin-left: 1.5ex; margin-right: 1.5ex; text-align: center !important; margin: 0px; background-color: white;',
          "header":'border-bottom: 1pt solid black; text-align: center !important; padding: 0.5ex 1.5ex; margin: 0px; background-color: white;',
          "rowgrouplabel": 'padding-top: 0.5ex; padding-right:1.5ex; padding-bottom:0.5ex;font-weight: bold; text-align: left !important;  white-space: nowrap; background-color: white;',
          "rowlabel": 'padding-top: 0.5ex; padding-right:1.5ex; padding-bottom:0.5ex; text-align: left !important; white-space: nowrap; background-color: white;',
          "footnote": 'font-size: smaller; padding: 0px; margin: 0px; text-align: left !important; ',
          "value": 'text-align: center !important; padding: 0.5ex 1.5ex; margin: 0px; background-color: white; white-space: nowrap;',
    }

empty_styles = {"table": '',
          "caption": '',
          "superheader": '',
          "header":'',
          "rowgrouplabel": '',
          "rowlabel": '',
          "footnote": '',
          "value": '',
    }


def get_styles(style):
    """
    Get the dictionary representing the available internal default styles

    :param style: what style to get. empty contains just keys with empty strings as values. Useful to
        override internal styles.
    :type style: dict
    :return: dictionary with internal class name as key and a string with css styles as values.
    :rtype: dict

    :Example:

    >>> from pyreportable import get_styles  
    >>> style = get_styles('default')
    """

    if style == "default":
        return default_styles
    elif style == "empty":
        return empty_styles
    else:
        raise Exception("No recognized styles")





# #############################################################################
# Copyright 2024 F. Hoffmann-La Roche
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# #############################################################################
import random
import os

import jinja2
import pandas as pd

from .intermediate_representation import df_to_intermediate_rep
from .styles import get_styles

dirpath = os.path.dirname(os.path.realpath(__file__))


def pandas_to_report_html(df, strat_numbers=None, caption=None, footer=None, customstyles=None, customcss=None, value_styles=None, styles='default', table_id=None, show_index=True):
    """
    Generate a nice html table from a pandas dataframe.

    :param df: Pandas dataframe. Can have simple or multi indexes and columns.
    :type df: pandas dataframe, mandatory
    :param strat_numbers: a dictionary where the key is a column name and value is a number to be displayed as N=xx
        below each column on the table header. If columns are multiindex, then the keys should be 
        a tuple with all the levels of the multiindex.
    :type strat_numbers: dictionary, optional
    :param caption: caption or title to set for the table.
    :type caption:  str, optional
    :param footer: footer for the table. Can be a string or a list of strings. If a list then every element
        will appear in a different line.
    :type footer: str or list of str, optional
    :param customstyles: dictionary where the keys are our internal classes and values are css styles. These styles
        will be appended to the styles. You can use an empty styles to completely override the defaults
        with your own styles.
    :type customstyles: dict, optional
    :param customcss: A string with css targeting classes or ids. It will be pre-pended to the table in a html <style> element
        without any further processing.
    :type customcss: str, optional 
    :param value_styles: css styles to apply to values, it will be appended to the "values" styles from the default styles. If a 2D structure
        each element represents the style for the corresponding value in the dataframe. If 1D each element is applied to
        every element on each row. If a string it will be applied to all elements.
    :type value_styles: str, list or list of lists, numpy 2d array or pandas dataframe
    :param styles: css styles to use. empty has no styles and is used to override all styles with yours.
    :type styles: str, {'default', 'empty'}
    :param table_id: id for the table, and as prefix for ids for every element on the table. If not provided,
        a random id will be generated internally. It can be an empty string.
    :type table_id: str, optional
    :param show_index: if False, dataframe indexes (row labels) will not be shown on the table.
    :type show_index: bool, optional
    :return: an object encapsulating the html representation of the table formatted in a nice way.
    :rtype: Pandas2HTMLSummaryTable 

    :Example:
    
    >>> from pyreportable import pandas_to_report_html  
    >>> html = pandas_to_report_html(df)

    """
    templateLoader = jinja2.FileSystemLoader(searchpath=dirpath)
    templateEnv = jinja2.Environment(loader=templateLoader)
    template_file = "template.html"
    jintemplate = templateEnv.get_template(template_file)

    styles = get_styles(styles)

    if customstyles:
        if type(customstyles) != dict:
            raise Exception("customstyles must be a dictionary")
        for k, v in styles.items():
            custom = customstyles.get(k)
            if custom:
                styles[k] = v + custom

    if footer and type(footer) != str:
        footer = "<br>".join(footer)

    if table_id is None:
        table_id = "pyt_" + str(random.getrandbits(16))

    if not strat_numbers:
        strat_numbers = dict()

    if not value_styles:
        value_styles = None
    elif type(value_styles) == str:
        value_styles = [value_styles] * len(df)
    elif hasattr(value_styles, '__iter__'):
        if len(value_styles) != len(df):
            raise Exception("value_styles must have the same name of rows and columns as the dataframe")
        if hasattr(value_styles[0], '__iter__') and type(value_styles[0]) != str:
            if len(value_styles[0]) != len(df.columns):
                raise Exception("value_styles must have the same name of rows and columns as the dataframe")
        else:
            value_styles = [[x]*len(df.columns) for x in value_styles]

    superheaders, tabheaders, numcols, bodyblocks = df_to_intermediate_rep(df, table_id, strat_numbers=strat_numbers, value_styles=value_styles, styles=styles, show_index=show_index)
    rendered = jintemplate.render(tabheaders=tabheaders,
                superheaders=superheaders,
                bodyblocks=bodyblocks,
                numcols=numcols, 
                caption=caption, 
                footer=footer, 
                styles=styles,
                customcss=customcss,
                tabid=table_id,
                )
    return  Pandas2HTMLSummaryTable(rendered)


class Pandas2HTMLSummaryTable:
    """
    A class to encapsulate the html representation
    of the summary table
    """
    def __init__(self, html):
        self.html = html

    def _repr_html_(self):
        return self.html

    def get_raw_html(self):
        """
        Get the raw HTML string representing the table

        :return: the html as a string
        :rtype: str 
        """
        return self.html




# #############################################################################
# Copyright 2024 F. Hoffmann-La Roche
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# #############################################################################
import datetime

import pandas as pd
import numpy as np

numeric_types = {np.dtype('int32'), np.dtype('int16'), np.dtype('int8'), np.dtype('uint8'), np.dtype('uint16'),
             np.int32, np.int16, np.int8, np.uint8, np.uint16, int, float,
            pd.Int8Dtype(), pd.Int16Dtype(), pd.Int32Dtype(), pd.UInt8Dtype(), pd.UInt16Dtype(),
             np.dtype('int64'), np.dtype('uint64'), np.dtype('uint32'), np.dtype('float'),
               np.int64, np.uint64, np.uint32, np.float64, pd.Int64Dtype(), pd.UInt32Dtype(), pd.UInt64Dtype(),
               pd.Float64Dtype(), pd.Float32Dtype()}
datetime_types = {datetime.datetime, np.datetime64, np.dtype('<M8[ns]'), np.datetime64}
categorical_types = {pd.core.dtypes.dtypes.CategoricalDtype, bool}


def detect_df_col_types(df):
    """
    Gets a dataframe and returns a dictionary with keys being column 
    names from the dataframe and value is the type:
    categorical, numerical or datetime
    """

    types = df.dtypes.values.tolist()
    columns = df.columns.values.tolist()

    results = dict()
    for colname, coltype in zip(columns, types):
        if coltype in categorical_types:
            results[colname] = "categorical"
            continue
        elif coltype in numeric_types:
            results[colname] = "numerical"
            continue
        elif coltype in datetime_types:
            results[colname] = "datetime"
        elif coltype == object:
            col = df[colname].dropna()
            if len(col):
                curtype = type(col.iloc[0])
                equal = np.array(col.apply(lambda x: type(x) == curtype))
                if not np.all(equal):
                    results[colname] = "categorical"
                    continue
            else:
                results[colname] = "numerical"
                continue
            if curtype in categorical_types:
                results[colname] = "categorical"
                continue
            elif curtype == str:
                results[colname] = "categorical"
            elif curtype in numeric_types:
                results[colname] = "numerical"
                continue
            elif curtype in datetime_types:
                results[colname] = "datetime"
            else:
                results[colname] = "categorical"
        else:
            results[colname] = "categorical"

    return results





import numpy as np
import pandas as pd

def get_sample_data():
    """
    Gets some data for testing and demo purposes

    :return: sample data
    :rtype: pandas dataframe
    """
    np.random.seed(42)
    data = {'gender': np.random.choice(['Male', 'Female'], 100, p=[0.6, 0.4]),
            'age': np.random.randint(20, 80, 100),
            'region': np.random.choice(['North', 'South', 'East', 'West'], 100, p=[0.3, 0.2, 0.1, 0.4]),
            'group': np.random.choice(['Control', 'Experimental'], 100, p=[0.4, 0.6]),
    }
    df = pd.DataFrame(data)
    df.loc[100] = [np.nan]*3 + ['Control']
    return df



# #############################################################################
# Copyright 2024 F. Hoffmann-La Roche
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# #############################################################################
from itertools import count


class TabSuperHeader:
    """
    Object to represent an item in the most external column indexes
    """
    def __init__(self, title, tabid, level, count, span=1, style="", top=False):
        self.title = title
        self.span = span
        self.style = style
        self.top = top
        self.level = level
        self.idnum = count
        self.classname = "superheader level_" + str(self.level) 
        self.idname = tabid + "_" + self.classname.replace(" ", "_") + "_" + str(self.idnum)

    def consolidate_styles(self):
        if self.span > 1: 
            self.style = 'border-bottom: 1pt solid black;' + self.style
        if self.top:
            self.style = 'border-top: 2pt solid black;' + self.style

class TabHeader:
    """
    Object to represent an item in the most internal column indexes
    """
    def __init__(self, title, tabid, count, n=None, style="", top=False):
        self.title = title
        self.n = n
        self.style = style
        self.classname = "header"
        self.idnum = count
        self.idname = tabid + "_header_" + str(self.idnum)
        self.idlabelname = tabid + "_headerlabel_" + str(self.idnum)
        self.idNname = tabid + "_headern_" + str(self.idnum)
        if top:
            self.style = 'border-top: 2pt solid black;' + self.style


class RowGroupLabel:
    """
    Object to represent an item in the most external row indexes
    """
    def __init__(self, title, tabid, level, left_padding, levelcounts, style=""):
        self.title = title
        self.left_padding = left_padding
        self.style = style
        self.level = level
        self.classname = "rowgrouplabel level_" + str(self.level)
        self.levelcounts = levelcounts
        self.tabid = tabid

    def consolidate_style(self):
        self.style = f"padding-left: {self.left_padding}ex;" + self.style
        # idname can be done only now after the final list of grouplabels has been pruned
        self.idnum = self.levelcounts[self.level]
        self.levelcounts[self.level] += 1
        self.idname = self.tabid + "_" + self.classname.replace(" ", "_") + "_" + str(self.idnum)


class RowLabel:
    """
    Object to represent an item in the most internal row indexes
    """
    instance_cnt = count(0)
    def __init__(self, title, left_padding, tabid, count, last_row=False, style=""):
        self.title = title
        self.left_padding = left_padding
        self.style = style
        self.last_row = last_row
        self.classname = "rowlabel"
        self.idnum = count
        self.idname = tabid + "_rowlabel_" + str(self.idnum)

    def consolidate_style(self):
        self.style = f"padding-left: {self.left_padding}ex;" + self.style
        if self.last_row:
            self.style = "border-bottom: 2pt solid black;" + self.style


class RowValue:
    """
    Object to represent an item in the row values
    """
    def __init__(self, value, tabid, rowcount, colcount, last_row=False, style=""):
        self.value = value
        self.style = style
        self.last_row = last_row
        self.classname = "rowvalue"
        self.rownum = rowcount
        self.colnum = colcount
        self.idname = tabid + "_value_row_" + str(self.rownum) + "_col_" + str(self.colnum)

    def consolidate_style(self):
        if self.last_row:
            self.style = "border-bottom: 2pt solid black;" + self.style


class Block:
    """
    Object to represent an Block (group of rows grouped by the most
    external row index)
    """
    titles = None
    rows = None

class Row:
    """
    Object to represent an Row
    """
    def __init__(self,label, values):
        self.label = label
        self.values = values



# #############################################################################
# Copyright 2024 F. Hoffmann-La Roche
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# #############################################################################
from great_tables import GT, html

from .table_summary import calculate_table_summary
from .reportable import pandas_to_report_html

# TODO:

def get_table_summary(df, strata=None, backend='native', show_n=True, show_overall=True, columns_labels=None, overall_name="Overall",
        columns_include=None, columns_exclude=None,
        rounding=1, categorical_functions=None, numerical_functions=None,
        categorical_missing_level='Missing', **kwargs):
    """
    Calculates a summary table for the pandas dataframe df and returns an object for nice display.

    :param df: pandas dataframe from which to calculate the table one
    :type df: pandas dataframe, mandatory
    :param strata: the name of a column in the dataframe to stratify the table one (columns)
    :type strata: str, optional
    :param backend: the backend used to display the summary, either 'native' or 'gt' (great_tables)
    :type backend: str, optional
    :param show_n: Show the number of observations on the column header
    :type show_n: bool, optional
    :param show_overall: Show the Overall column. By default True. If False it will take effect only if strata is defined, otherwise ignored
    :type show_overall: bool, optional
    :param columns_labels: A dictionary defining labels for the columns. Keys should be the column name and values a string with the label. Non existing
        columns will be ignored.
    :type columns_labels: dict, optional
    :param overall_name: name for the column Overall
    :type overall_name: string, optional
    :param columns_include: columns to include in the report, also determines the order of columns in the report
    :type columns_include: list, optional
    :param columns_exclude: columns to exclude from the report
    :type columns_exclude: list, optional
    :param rounding: number of decimal points to show, by default 1.
    :type rounding: int, optional
    :param categorical_functions: if a string, one of the presets for categorical summarization will be applied, by default n_percent. If a tuple or list, 
        the first element must be a function to apply to categorical functions, the second element must be a string or number with the value to report if there are no 
        elements in a categorical level (for example '(0%)' for n_percent). If None those will be reported as nan.
    :type categorical_functions: str,  list or tuple, optional
    :param numerical_functions: if a string, one of the presets for numerical summarization will be applied, the default one is 'meansd_medianq1q3_minmax_missing. 
        If a dictionary it should have a label (as it should appear in the rows index) and as a values 
        functions to apply to the numerical columns of the dataframe. Multiple pairs of labels and functions are supported. 
    :type numerical_functions: str or dict, optional
    :param categorical_missing_level: if a categorical column has NAs, they will be replaced by the string indicated here, by default 'Missing'. That will create a new level 
        in the category. If set to None, the NAs will not be replaced.
    :type categorical_missing_level: str, optional
    :param kwargs: keyword arguemnts to pass to the pandas_to_report_html function or the great_tables.GT constructor. See the documentation for those for further details.
    :return: An object with the html representation of the table
    :rtype: Pandas2HTMLSummaryTable if backend is native or great_tables.GT if backend is gt
        
    :Example:
    
    >>> from pysummaries import get_table_summary
    >>> tone = get_table_summary(df, strata="age_groups")

    """
    if backend not in ('native', 'gt'):
        raise Exception(f"Available backends are 'native' or 'gt', got {backend}")

    tone, strat_numbers = calculate_table_summary(df, strata=strata, show_overall=show_overall, columns_labels=columns_labels, overall_name=overall_name, rounding=rounding, 
            columns_include=columns_include, columns_exclude=columns_exclude,
            categorical_functions=categorical_functions, numerical_functions=numerical_functions,
            categorical_missing_level=categorical_missing_level)  
    if backend == 'native':
        if show_n:
            tone_html = pandas_to_report_html(tone, strat_numbers=strat_numbers, **kwargs) 
        else:
            tone_html = pandas_to_report_html(tone, strat_numbers=None, **kwargs) 
    elif backend == 'gt':
        tone_html = GT(tone.reset_index(), rowname_col="level_1", groupname_col="level_0", **kwargs)
        # Ns 
        if show_n:
            col_ns = {k:html(k+f"<br>(N={v})") for k,v in strat_numbers.items()}
            tone_html = tone_html.cols_label(**col_ns)
    return tone_html




# #############################################################################
# Copyright 2024 F. Hoffmann-La Roche
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# #############################################################################

from datetime import datetime, timedelta, date
import unittest
import os
import sys
import shutil
import pickle

import pandas as pd
import numpy as np


class TestBasic(unittest.TestCase):
    """
    Test suite for pysummaries.
    """

    def _prepare_data(self):

        self.sample_data = pysummaries.get_sample_data()

        self.table_id = "pyt_1234"
        with open(os.path.join(script_folder, 'test_data.pkl'), 'rb') as f:
            self.test_data = pickle.load(f)


    def setUp(self):

        self._prepare_data()

    def test_simple_table_summary_df(self):
        sum_table, strat_nums = pysummaries.calculate_table_summary(self.sample_data, strata='group')
        sum_table_test, strat_nums_test = self.test_data['summary_table_df'] 
        self.assertTrue(sum_table.equals(sum_table_test))
        self.assertTrue(strat_nums['Overall']==strat_nums_test['Overall'])

    def test_simple_table_summary_native(self):
        sum_table_html = pysummaries.get_table_summary(self.sample_data, strata='group', backend='native', table_id=self.table_id).get_raw_html()
        sum_table_html_test = self.test_data['summary_table_html'] 
        self.assertTrue(sum_table_html==sum_table_html_test)

    def test_simple_table_summary_gt(self):
        sum_table_html = pysummaries.get_table_summary(self.sample_data, strata='group', backend='gt', id=self.table_id).as_raw_html()
        sum_table_html_test = self.test_data['summary_table_gt'] 
        self.assertTrue(sum_table_html==sum_table_html_test)

if __name__ == '__main__':

    import sys

    script_folder = os.path.split(os.path.realpath(__file__))[0]
    repo_folder = os.path.split(script_folder)[0]

    if "--inplace" in sys.argv:
        sys.path.insert(0, repo_folder)
        sys.argv.remove('--inplace')

    import pysummaries

    print("package location:", pysummaries.__file__)

    unittest.main()



# #############################################################################
# Copyright 2024 F. Hoffmann-La Roche
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# #############################################################################

import os
from setuptools import setup, find_packages


long_description="""
PySummaries is a Python package to easily produce table summarizations
from pandas dataframes.

For more detailed information, please look at the [documentation](https://genentech.github.io/pysummaries/)
or our [project homepage](https://github.com/Genentech/pysummaries)
"""

setup(
    name="pysummaries",
    version='0.0.1b1',
    author="Otto Fajardo",
    author_email="pleasecontactviagithub@notvalid.com",
    description="Produce table summaries from pandas dataframes",
    license="Apache License Version 2.0",
    keywords="pandas tableone table1 python table_summary tables",
    url="https://github.com/Genentech/pysummaries",
    packages=find_packages(),
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=['pandas>=2.0.0', 'great-tables>=0.11.0', 'jinja2'],
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",
        "Environment :: Console",
    ],
)



# #############################################################################
# Copyright 2024 F. Hoffmann-La Roche
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# #############################################################################

"""
Test if the version in pysummaries __init__ matches the version in setup.py
"""

import os
import sys
import re

script_folder = os.path.split(os.path.split(os.path.realpath(__file__))[0])[0]
sys.path.insert(0, script_folder)
sys.path.insert(1,os.path.join(script_folder, "docs"))

import pysummaries

pysummaries_version = pysummaries.__version__

with open(os.path.join(script_folder, "setup.py")) as h:
    content = h.read()

raw = re.findall("version=\'.*?\'", content)
setup_version = raw[0].replace("version=", "")
setup_version = setup_version.replace("'", "")

#with open(os.path.join(script_folder, "CITATION.cff")) as h:
    #content = h.readlines()
#raw = [x for x in content if x.startswith("version")]
#cff_version = raw[0].replace("version:", "").strip()

print("testing if module and setup versions match")
assert(pysummaries_version == setup_version)
#print("testing if documentation and setup versions match")
#assert(conf.release == setup_version)
#print("testing if cff and setup versions match")
#assert(cff_version == setup_version)
print("all versions match!")



